Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.AKMSensorConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device = config.device;

        var devInfo = dSS.util.parseProductID(this.device.get('productID'));
        this.deviceType = devInfo[0];

        this.disableFlag = false;

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.sensorSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 173,
            mode: 'local',
            fieldLabel: dSS._("Sensor function"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [
                        0, // App-Mode
                        dSS._(dSS.config.AKMInputTypeStrings[0])
                    ],
                    [
                        1, // Presence
                        dSS._(dSS.config.AKMInputTypeStrings[1])
                    ],
                    [
                        4, // Twilight
                        dSS._(dSS.config.AKMInputTypeStrings[4])
                    ],
                    [
                        5, // Movement
                        dSS._(dSS.config.AKMInputTypeStrings[5])
                    ],
                    [
                        7, // Smoke
                        dSS._(dSS.config.AKMInputTypeStrings[7])
                    ],
                    [
                        8, // Wind
                        dSS._(dSS.config.AKMInputTypeStrings[8])
                    ],
                    [
                        9, // Rain
                        dSS._(dSS.config.AKMInputTypeStrings[9])
                    ],
                    [
                        10, // Sunlight
                        dSS._(dSS.config.AKMInputTypeStrings[10])
                    ],
                    [
                        17, // Sun Protection
                        dSS._(dSS.config.AKMInputTypeStrings[17])
                    ],
                    [
                        18, // Frost
                        dSS._(dSS.config.AKMInputTypeStrings[18])
                    ],
                    [
                        11, // Thermostat
                        dSS._(dSS.config.AKMInputTypeStrings[11])
                    ],
                    [
                        13, // Window contact
                        dSS._(dSS.config.AKMInputTypeStrings[13])
                    ],
                    [
                        14, // Door contact
                        dSS._(dSS.config.AKMInputTypeStrings[14])
                    ],
                    [
                        16, // Garage door contact
                        dSS._(dSS.config.AKMInputTypeStrings[16])
                    ],
                    [
                        19, // Heating system
                        dSS._(dSS.config.AKMInputTypeStrings[19])
                    ],
                    [
                        20, // Heating mode
                        dSS._(dSS.config.AKMInputTypeStrings[20])
                    ]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            height: 35,
            scope: this,
            hidden: ((this.deviceType === dSS.config.deviceTypeNum.UMR) &&
                     (this.group != dSS.config.groups.JOKER)),
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.sensorSelection
                    ]
                }
            ]
        });

        dSS.devprops.AKMSensorConfigForm.superclass.constructor.apply(this, arguments);

        dSS.MessageBus.addListener('jokerconfigformchanged',
            this.updateVisibility, this);

        dSS.MessageBus.addListener('pushbuttonstdformchanged',
            this.handleAvailability, this);

        this.on('loadcomplete', function() {
                this.sensorSelection.on('select', function() {
                    this.fireEvent('formchanged');
                },
                this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
                dSS.MessageBus.removeListener('jokerconfigformchanged',
                   this.updateVisibility, this);

                dSS.MessageBus.removeListener('pushbuttonstdformchanged',
                   this.handleAvailability, this);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.AKM_BINARY_INPUTS) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.sensorSelection.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                var inputs = data.json.result.inputs;
                if (inputs.length < 1) {
                    this.sensorSelection.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                }

                if (inputs.length > 0) {
                    this.loadedValue = inputs[0].inputType;
                } else {
                    this.loadedValue = 0;
                }
                this.sensorSelection.setValue(this.loadedValue);
                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function() {
            var value = this.sensorSelection.getValue();
            var dSUID = this.device.get('dsuid');
            var loaded = this.loadedValue;
            if (value === loaded) {
                this.fireEvent('formsubmitted');
                return;
            }

            dSS.util.genericRequest({
                url: '/json/device/setBinaryInputType',
                params:
                {
                    dsuid: dSUID,
                    index: 0,
                    type: value
                },
                failureMsg: dSS._("Could not save sensor function"),
                scope: this,
                evt: 'formsubmitted'
            },
            function (ok)
            {
                if (!ok) {
                    return;
                }
                var device = dSS.data.Loader.getDeviceStore().getById(dSUID);
                if (device === undefined) {
                    return;
                }
                var inputs = device.get('binaryInputs');
                var i;
                for (i = 0; i < inputs.length; i++) {
                    if (inputs[i].inputType == loaded) {
                        inputs[i].inputType = value;
                        break;
                    }
                }
                device.set('binaryInputs', inputs);
                device.commit();
            });
        },
        this);
    },
    updateVisibility: function(group) {
        this.doLayout();
        if ((this.deviceType === dSS.config.deviceTypeNum.UMR) &&
            (this.device.get('dSUIDIndex') === 0)) {
            if (group === dSS.config.groups.JOKER) {
                if (!this.isVisible()) {
                    this.show();
                }
            } else {
                if (this.isVisible()) {
                    this.hide();
                }
            }
        }

    },
    handleAvailability: function(setting) {
        if (this.deviceType === dSS.config.deviceTypeNum.UMR) {
            if (setting === dSS.devprops.PBConfig.PUSHBUTTON_SENSOR) {
                // work around Ext weirdness
                if (this.el.dom) {
                   this.enable();
                }
            } else {
                if (this.el.dom) {
                    this.disable();
                }
            }
        }
    }
});

Ext.reg('dssakmsensorconfigform', dSS.devprops.AKMSensorConfigForm);

